// Pseudo 'C' code to emulate Solar Quest sound board.
//
// The routine 'play( WaveForm.wav)' is used to play a .WAV file from
// it's start.  If the file is already being played, 'play()' will
// stop the current playing of the file, and restart it from the begining.
//
// The routine 'loop( WaveForm.wav)' is used to play a .WAV file continuously
// by looping the file from end back to start.  This will be done until the
// routine 'stop( WaveForm.wav)' is called.
//
// The routines: play(), loop() and stop() are to be defined by you.
//
// The routine 'startSquareWave( uint freq, uint vol)' must also be written.
// The 'freq' is given in Hertz, and the volume is the volume given in the files:
// "Music0.wav" through "Music6.wav"
//
// The routine 'stopSquareWave( void)' stop the square wave from playing.

typedef unsigned char	uchar;
typedef unsigned int		uint;

extern void play( char *);			// play a wave file from the begining
extern void loop( char *);			// loop a wave file
extern void stop( char *);			// stop the looping of a wave file

extern void startSquareWave( uint freq, uint vol);	// generate a square wave
extern void stopSquareWave( void);						// turn off the square wave

#define	OTB_DATA			0x80
#define	OTB_CLOCK		0x10
#define	OTB_SR1LATCH	0x02
#define	OTB_SR2LATCH	0x01

#define	SR1_LOUDEXP		0x80
#define	SR1_SOFTEXP		0x40
#define	SR1_THRUST		0x20
#define	SR1_FIRE			0x10
#define	SR1_CAPTURE		0x08
#define	SR1_NUKE			0x04
#define	SR1_PHOTON		0x02

#define	SR2_MUSIC		0x8000
#define	SR2_VOLUME		0x7000
#define	SR2_PITCH		0x0FFF

uchar	OldOutReg;							// keep a copy of previous OUT value
uchar	SR1reg, OldSR1reg;				// keep a copy of shift register and previous value
uint	SR2reg, OldSR2reg;				// keep a copy of shift register and previous value
uint	TSreg;								// Temporary shift register

// Called to initialize the sound emulation.
//
// outReg = New value of the CCPU's OUT register.

void initSounds( uchar outReg)
{
	SR1reg = SR1_LOUDEXP | SR1_SOFTEXP | SR1_THRUST | SR1_FIRE | SR1_CAPTURE | SR1_PHOTON;
	SR2reg = 0;

	OldOutReg = outReg;					// synchronize the register variables
	OldSR1reg = SR1reg;
	OldSR2reg = SR2reg;
}

// This routine is to be called each time the CCPU's OUT register's
// bits 7, 4, 3, 2, 1 or 0 change states.
//
// outReg = New value of the CCPU's OUT register.

void updateSounds( uchar outReg)
{
	uchar outDiff;							// changed bits
	uchar	outHigh;							// changed bits that have just gone high
//	uchar	outLow;							// changed bits that have just gone low (not used)

	uchar	sr1Diff;							// changed bits	
	uchar	sr1High;							// changed bits that have just gone high
	uchar	sr1Low;							// changed bits that have just gone low

	uint	sr2Diff;							// changed bits	
	uint	sr2High;							// changed bits that have just gone high
	uint	sr2Low;							// changed bits that have just gone low

	outDiff = outReg ^ OldOutReg;		// get bits that have changed state
	outHigh = outDiff & outReg;		// get bits that have just gone high
//	outLow = outDiff & ~outReg;		// get bits that have just gone low (not used)

	// check if TSR is being clocked

	if (outHigh & OTB_CLOCK)
	{	TSreg >>= 1;								// shift TSreg right one
		TSreg |= (outReg & OTB_DATA) << 8;	// shift in new bit
	}

	// check if SR1 is being latched

	if (outHigh & OTB_SR1LATCH)
	{	SR1reg = TSreg >> 8;								// save new SR1 value

		// check for new SR1 sounds

		sr1Diff = SR1reg ^ OldSR1reg;					// get diff's
		sr1High = sr1Diff & SR1reg;					// get bits that just went high
		sr1Low = sr1Diff & ~SR1reg;					// get bits that just went low

		if (sr1Low & SR1_LOUDEXP)
			play( "LoudExp.wav");

		if (sr1Low & SR1_SOFTEXP)
			play( "SoftExp.wav");

		// On the real hardware, if the Thrust sound is in decay, and the thrust button
		// is pressed, the Thrust will start to climb to full volume starting from
		// the current value.
		//
		// Chances are that just starting over with the attack wave, might
		// sound fine, Especially if the release waveform is allowed
		// to continue playing while the attack waveform is played.
		// (That's what's shown here.)

		if (sr1Low & SR1_THRUST)
		{	play( "ThrustAttack.wav");

			// wait for the playing of "TankFireAttack.wav" to finish...

			loop( "ThrustSustain.wav");
		}

		if (sr1High & SR1_THRUST)
		{	stop( "ThrustSustain.wav");
			play( "ThrustRelease.wav");
		}

		if (sr1Low & SR1_FIRE)
			play( "Fire.wav");

		if (sr1Low & SR1_CAPTURE)
			play( "Capture.wav");

		if (sr1High & SR1_NUKE)
			loop( "Nuke.wav");

		if (sr1Low & SR1_NUKE)
			stop( "Nuke.wav");

		if (sr1Low & SR1_PHOTON)
			play( "Photon.wav");

		OldSR1reg = SR1reg;
	}

	// check if SR2 is being latched

	if (outHigh & OTB_SR2LATCH)
	{	SR2reg = TSreg;									// save new SR2 value

		// check for new SR1 sounds

		sr2Diff = SR2reg ^ OldSR2reg;					// get diff's
		sr2High = sr2Diff & SR2reg;					// get bits that just went high
		sr2Low = sr2Diff & ~SR2reg;					// get bits that just went low

		if ((sr2High & SR2_MUSIC) || ((SR2reg & SR2_MUSIC) && (sr2Diff & (SR2_PITCH | SR2_VOLUME))))
		{
			// start playing a square wave, if a previous square wave was playing, stop it
			// and start playing this one.

			startSquareWave( 56818 / (4096 - (SR2reg & SR2_PITCH)), (SR2reg & SR2_VOLUME) >> 12);
		}

		if (sr2Low & SR2_MUSIC)
			stopSquareWave();

		OldSR2reg = SR2reg;
	}
		 
	OldOutReg = outReg;									// save new OUT register
}
